{% extends "django_mailman3/profile/base.html" %}
{% load i18n %}
{% load static %}
{% load bootstrap_tags %}


{% block user_profile_content %}
    <div class="row">
        <form method="post" action="" class="form-horizontal">
            {% csrf_token %}
            <p>
                {% blocktrans trimmed %}
                Are you sure you want to delete your account?
                This will remove your account along with all your subscriptions.
                {% endblocktrans %}
            </p>
            <button type="submit" class="btn btn-danger">{% trans 'Delete Account' %}</button>
        </form>
    </div>
{% endblock %}
